require "spec_helper"
require 'active_record'
require "ohm"

include WeatherModel

RSpec.describe WeatherModel do

  before do
    # start an ohm
    WeatherModel.set_ohm Redic.new("redis://127.0.0.1:6379/5")
    # run migration
    Dir[File.dirname(__FILE__) + '/../db/migrate/*.rb'].each { |file| require  file }
    # drop table
    FileUtils.rm 'weather-model-test.sqlite3'
    ActiveRecord::Base.establish_connection(
      :adapter => 'sqlite3',
      :database => 'weather-model-test.sqlite3'
    )
    ActiveRecord::Migration[5.0].subclasses.each do |migrate|
      migrate.migrate(:up)
    end
  end

  it "has a version number" do
    expect(WeatherModel::VERSION).not_to be nil
  end

  describe 'test factory_girl' do
    before do
      @obj = create(:city_forecast)  
    end

    it 'Test factory_girl' do
      expect(@obj.class).to eq(Forecast::CityForecast), 'factory_girl 生成成功'
    end
    
  end

  it 'Forecast' do
    @expire = 1
    Forecast::CityForecast::Ohm.expire @expire
    mysql_obj = Forecast::CityForecast.create({
          datetime: '2017-05-16',
          cityname: 'shanghai',
          temp_high: '100',
          temp_low: '0',
          weather_text: '酷热严寒',
          weather_pic: 'http://xxx.com',
          win_dir: '东南西北风',
          win_speed: '1级',
        })
    expect(Forecast::CityForecast.count).to eq(1)
    expect(Forecast::CityForecast::Ohm.all.count).to eq(1)
    expect(Forecast::CityForecast::Ohm.all.first.temp_low).to eq(mysql_obj.temp_low.to_s)
    expect(Forecast::CityForecast::Ohm.all.first.get_ttl).to eq(@expire)
    # 'ohm 对象 同步 mysql对象更新'
    mysql_obj.update(temp_low: 99)
    expect(Forecast::CityForecast::Ohm.all.first.temp_low).to eq('99'), 'ohm 对象 同步 mysql对象更新'
    sleep 1.5
    expect(Forecast::CityForecast::Ohm.all.count).to eq(0), 'TTL过期对象删除成功'
  end

  it 'aqi_forecast' do
    @aqi_forecast = Aqi::Forecast.create(
      datetime: Time.now,
      prompt: '',
      list: [
        {
          "period"=>"今天夜间（20时—06时）",
          "aqi"=>"35-55",
          "level"=>"优到良",
          "pripoll"=>"PM10"
        }, {
          "period"=>"明天上午（06时—12时）",
          "aqi"=>"35-55",
          "level"=>"优到良",
          "pripoll"=>"PM10"
        },
      ]
    )
    expect(Aqi::Forecast::Ohm.all.first).not_to be_nil, 'json_column 正常'
  end

  after do
    ActiveRecord::Migration[5.0].subclasses.each do |migrate|
      migrate.migrate(:down) 
    end 
  end

end
